#!/bin/bash

fail() { [ $# -eq 0 ] || echo "$@"; exit 1; }

if [ "${TAG}" == "release" ]; then
    TAG="release-${SERIAL}"
fi

src_d="/srv/maas-images/ephemeral/daily/${SUITE}/${SERIAL}"
final_out_d="/srv/maas-images/ephemeral/releases/${SUITE}/${TAG}"

[ -e ${src_d} ] ||
    fail "Source ${src_d} does not exist"

[ -e ${final_out_d} ] &&
    fail "Serial has already been promoted"

mkdir -p "${final_out_d}" &&
    rsync -a ${src_d}/ ${final_out_d} &&
    echo "Copied bits to final location ${final_out_d}" ||
    fail "Unable to copy build bits to final location"

# Generate the Query2 tree
export maas_branch="${MAAS_BRANCH:-http://bazaar.launchpad.net/~smoser/maas/maas.ubuntu.com.images-ephemeral}"
src_tree="${WORKSPACE}/maas_src"
bzr branch "${maas_branch}" "${src_tree}"
${src_tree}/tree2query \
    --commit-msg "Build ${BUILD_ID}" \
    --namespace maas \
    /srv/maas-images

