#!/usr/bin/python

#
# This reads the Jenkins json results and computes commands to run
# against the ISOTracker

import argparse
import json
import urllib2
import sys
import time

from pprint import pprint as pp

parser = argparse.ArgumentParser()
parser.add_argument('--host',
        action="store",
        default="localhost:8080",
        help="Host address to hit")
parser.add_argument('--test',
        action="store",
        default=None,
        help="Test suite to parse")
parser.add_argument('--suite',
        action="store",
        default="raring",
        required=True,
        help="Which suite are we working with")
parser.add_argument('--milestone',
        action="store",
        default=None,
        required=True,
        help="Which milestone?")
parser.add_argument('--out',
        action="store",
        default=None,
        required=True,
        help="Output file of commands")
parser.add_argument('--serial',
        action="store",
        default=None,
        required=True,
        help="Serial to use")

opts = parser.parse_args()

sys.stderr.write("Using host: %s\n" % opts.host)

if not opts.test:
    sys.stderr.write("Must define test with --test\n")
    sys.exit(3)


# Translations for values.
translations = {
    'us-east-1': "US-East",
    'ap-northeast-1': "Asia-Pacific-NorthEast",
    'ap-southeast-1': "Asia-Pacific-SouthEast",
    'ap-southeast-2': "Asia-Pacific-SouthEast-Australia",
    'eu-west-1': "Europe",
    'eu-central-1': 'Europe-Central',
    'sa-east-1': "South-America-East-1",
    'us-west-1': "US-West-1",
    'us-west-2': "US-West-2",
    'simple-user-data': "EC2 User Data",
    'multi-instance': "EC2 Multiple Instances Run",
    'all-types': "EC2 Multiple Instances Run",
    'instance-store': "Instance",
    'ebs': "EBS",
    'ebs-ssd': "EBS-SSD",
    'ebs-io1': "EBS-IO1",
    'hvm:EBS': "HVM",
    'hvm:EBS-SSD': "HVM-SSD",
    'hvm:EBS-IO1': "HVM-IO1",
    'hvm:Instance': "HVM Instance",
    }

# Get the ami lookup
url = "http://cloud-images.ubuntu.com/query/%s/server/daily.txt" % \
    opts.suite.lower()
print "Fetching AMI listing..."
response = urllib2.urlopen(url)
ami_lookup = {}
serial = None
for line in str(response.read()).splitlines():

    if len(line.split()) == 9:
        suite, _, _, serial, disk, arch, region, ami, virt = \
            line.split()
    else:
        suite, _, _, serial, disk, arch, region, ami, _, virt = \
            line.split()

    # Make sure we're reading the write file
    if suite != opts.suite.lower():
        raise Exception("Suite does not match suite query!")

    # Make sure we're using the right serial
    if opts.serial != serial:
        continue

    if region not in ami_lookup:
        ami_lookup[region] = {}

    if arch not in ami_lookup[region]:
        ami_lookup[region][arch] = {}

    if virt not in ami_lookup[region][arch]:
        ami_lookup[region][arch][virt] = {}

    if disk not in ami_lookup[region][arch][virt]:
        ami_lookup[region][arch][virt][disk] = ami


# Get the json for parsing
url = "http://%s/job/%s/api/json?pretty=true" % (opts.host, opts.test)
print "Fetching results..."
response = urllib2.urlopen(url)
json_out = response.read()
json_d = json.loads(json_out)
results= json_d['activeConfigurations']

pass_count, fail_count = 0, 0
status = []
for result in results:
    r = {}
    url = result['url']
    r['URL'] = url

    # Extract the elements
    for item in (url.rstrip('/').split('/'))[-1].split(','):
        k, v = item.split('=')
        v = v.replace('/','')
        r[k] = v

    # Assume we're PV, unless otherwise stated
    if 'VIRT_TYPE' not in r or r['VIRT_TYPE'] == 'paravirt':
       r['VIRT_TYPE'] = 'paravirtual'

    # Get the AMI id
    ami = ami_lookup[r['REGION']][r['ARCH']][r['VIRT_TYPE']][r['STORAGE']]
    r['AMI'] = ami
    r['SERIAL'] = serial
    r['STORAGE'] = translations[r['STORAGE']]
    r['REGION'] = translations[r['REGION']]
    r['MILESTONE'] = "%s %s" % (opts.suite.title(), opts.milestone.title())

    if r['VIRT_TYPE'] == 'hvm':
        r['STORAGE'] = translations['hvm:%s' % r['STORAGE']]

    # Label the build
    r['PRODUCT'] = '"Ubuntu Server EC2 %(STORAGE)s (%(REGION)s) %(ARCH)s"' \
        % r

    # Make a comment
    r['COMMENT'] = \
        "[ %(SERIAL)s %(ARCH)s/%(STORAGE)s/%(TEST)s/%(REGION)s ]" \
        % r

    # Translate the values
    for k, v in r.iteritems():
        if v in translations:
            r[k] = translations[v]

    if result['color'] == 'blue':
        r['PASS'] = 'Passed'
        pass_count += 1
    else:
        r['PASS'] = 'Failed'
        fail_count += 1

    status.append(r)

print "Failed Count: %s" % fail_count
print "Pass Count: %s" % pass_count

with open(opts.out, 'w') as f:
    for item in status:
        cmd = \
"""tracker_update_result -u ${API_USER} -p ${API_KEY} --debug """ \
""" "--comment=%(COMMENT)s" "%(MILESTONE)s" """ \
""" %(PRODUCT)s "%(TEST)s" %(AMI)s %(PASS)s\n""" % item
        f.write(cmd)

f.close()

print "Wrote %s" % opts.out
